/*
 *  authorise  -  sendmail authorisation program
 *
 *	Given a channel name, sender address and recipient address or
 *	host, it matches these against entries in an authorisation
 *	file to see if permission is granted to send the mail.
 *
 *	If successful, the mailer interface is called; else the 
 *	appropriate error status is returned to sendmail.
 *
 *	Written by Jim Crammond.	<jac@ic.doc>	5/87
 *	Converted for full regular expressions
 *	Andrew Findlay, Feb 1988.
 *	andrew@uk.ac.brunel.me
 */
#include <stdio.h>
#include <sysexits.h>
#include <ctype.h>

#define AUTHFILE	"/usr/lib/sendmail.auth"
#define LOGFILE		"/usr/lib/sendmail.auth_log"

#define POSTMASTER	"postmaster@unido.uucp"

#define	CHANSIZE	64
#define	ADDRSIZE	256
#define LINESIZE	1024

char	a_chan[CHANSIZE];
char	a_ok[ADDRSIZE];
char	a_from[ADDRSIZE];
char	a_to[ADDRSIZE];

char	*authfile = AUTHFILE;

main(argc, argv)
int	argc;
char	*argv[];
{
	char	*progname, *channel;
	char	*from_addr, *to_addr;
	FILE	*afp, *logf, *fopen();
	char	line[LINESIZE];
	int	cnt=0, nfields;
	register char *p;
	char	*index();
	int	gotmatch = 0;


	progname  = *argv++;

	if (argc < 4)
	{	printf("usage: %s [-f authfile] channel from to command [args]\n", progname);
		exit(EX_USAGE);
	}

	if (argv[0][0] == '-' && argv[0][1] == 'f')
	{	authfile = argv[1];
		argc -= 2;
		argv += 2;
	}

	if (argc < 4)
	{	printf("usage: %s [-f authfile] channel from to command [args]\n", progname);
		exit(EX_USAGE);
	}

	channel   = *argv++;
	from_addr = *argv++;
	to_addr   = *argv++;

	if ((afp = fopen(authfile, "r")) == NULL)
	{	printf("warning: cannot open authorisation file\n");
		gotmatch = 1;
	}

#ifdef LOGFILE
	if ((logf = fopen(LOGFILE, "a")) == NULL)
	{	printf("warning: cannot open log file\n");
	}
	else
	{
		if (afp == NULL) fprintf(logf, "Cannot open authfile '%s'\n",
						authfile );
		fprintf(logf, "%s\t%s\t%s\t", channel, from_addr, to_addr );
	}
#endif LOGFILE

	while (!gotmatch && fgets(line, sizeof(line), afp))
	{	cnt++;
		if ((p = index(line, '\n')) != NULL)
			*p = '\0';
		if ((p = index(line, '#')) != NULL)
			*p = '\0';
		
		for (p=line; *p == ' ' && *p == '\t'; p++)
			;

		if (*p == '\0')
			continue;

		nfields = sscanf(p, "%s %s %s %s", a_chan, a_ok, a_from, a_to);

		if (nfields != 4)
		{	printf("warning: line %d ignored: \"%s\"\n", cnt, line);
			continue;
		}

		if (strcmp(a_ok, "OK") && strcmp(a_ok, "BLOCK") )
		{
			printf("warning: line %d is not OK or BLOCK: \"%s\"\n",
				cnt, line);
			continue;
		}

		if (strcmp(channel, a_chan) == 0 &&
		    match(from_addr, a_from) && match(to_addr, a_to))
		{	if (strcmp(a_ok, "BLOCK") == 0)
				gotmatch = -1;
			else
				gotmatch = 1;
		}

#ifdef DEBUG
		printf("%d: %s - %s\n", cnt, line,
			gotmatch ? "matched" : "no match");
#endif DEBUG
	}

#ifdef LOGFILE
	if (logf != NULL) {
		fprintf(logf, "%s %d\n", (gotmatch<=0)?"BLOCKED":"OK", cnt );
		fclose(logf);
	}
#endif LOGFILE

	if (gotmatch <= 0)
	{	printf("%s:\n\t%s is not authorised to send to\n\t%s\n"
,
			progname, from_addr, to_addr);
		printf("\nFor information, mail to %s\n\n", POSTMASTER );
		exit(EX_NOPERM);
	}

	execv(argv[0], argv);

	printf("%s: cannot exec %s\n", progname, argv[0]);
	exit(EX_UNAVAILABLE);
}


/*
 *  MATCH  --  match the strings s1 and s2.
 *		s2 can contain wildcards and lists
 */
match(s1, s2)
char	*s1, *s2;
{
	char *s;	/* for re_comp returns and general messing */
	int n;		/* for re_exec returns */

	char m_address[ADDRSIZE], m_pattern[ADDRSIZE];

	char *re_comp();
	int re_exec();

	/* Make copies of strings, converting to lower case */
	for (s=m_address; *s1; s1++, s++)
		*s = isupper(*s1) ? tolower(*s1) : *s1;
	*s = '\0';

	for (s=m_pattern; *s2; s2++, s++)
		*s = isupper(*s2) ? tolower(*s2) : *s2;
	*s = '\0';

	s = re_comp(m_pattern);

	if (s != 0) {
		printf( "Bad regular expression: '%s'\n", m_pattern );
		return(0);
	}

	n = re_exec(m_address);
	if (n < 0) {
		printf( "Bad regular found\n" );
		return(0);
	}

	return(n);
}

